import "graphile-build";
import type { PgTypeCodec } from "@dataplan/pg";
/**
 * Metadata for a specific PgTypeCodec
 */
export interface PgTypeCodecMeta {
    /**
     * Given a `situation` such as 'input', 'output', 'patch', etc. returns the
     * name of the GraphQL type to use for this PgTypeCodec.
     */
    typeNameBySituation: {
        [situation: string]: string;
    };
}
/**
 * A map from PgTypeCodec to its associated metadata.
 */
export type PgTypeCodecMetaLookup = Map<PgTypeCodec<any, any, any>, PgTypeCodecMeta>;
/**
 * Creates an empty meta object for the given codec.
 */
export declare function makePgTypeCodecMeta(_codec: PgTypeCodec<any, any, any>): PgTypeCodecMeta;
/**
 * Given the input object, this function walks through all the pgSources and
 * all their codecs and relations and extracts the full set of reachable
 * codecs.
 *
 * Memoized for performance, using a WeakMap.
 */
export declare function getCodecMetaLookupFromInput(input: GraphileBuild.BuildInput): PgTypeCodecMetaLookup;
//# sourceMappingURL=inputUtils.d.ts.map