import "graphile-build";
import "./PgTablesPlugin.js";
import "../interfaces.js";
import "graphile-config";
import type { PgTypeCodec, PgTypeColumn } from "@dataplan/pg";
declare global {
    namespace GraphileBuild {
        interface Inflection {
            /**
             * Given a columnName on a PgTypeCodec's columns, should return the field
             * name to use to represent this column (both for input and output).
             *
             * @remarks The method beginning with `_` implies it's not ment to
             * be called directly, instead it's called from other inflectors to give
             * them common behavior.
             */
            _columnName(this: GraphileBuild.Inflection, details: {
                codec: PgTypeCodec<any, any, any, any>;
                columnName: string;
                skipRowId?: boolean;
            }): string;
            /**
             * Takes a codec and the list of column names from that codec and turns
             * it into a joined list.
             */
            _joinColumnNames(this: GraphileBuild.Inflection, codec: PgTypeCodec<any, any, any, any>, names: readonly string[]): string;
            /**
             * The field name for a given column on that pg_class' table type. May
             * also be used in other places (e.g. the Input or Patch type associated
             * with the table).
             */
            column(this: GraphileBuild.Inflection, details: {
                columnName: string;
                codec: PgTypeCodec<any, any, any>;
            }): string;
        }
        interface ScopeInputObject {
            isInputType?: boolean;
            isPgPatch?: boolean;
            isPgBaseInput?: boolean;
            isPgRowType?: boolean;
            isPgCompoundType?: boolean;
            pgColumn?: PgTypeColumn;
        }
    }
}
export declare const PgColumnsPlugin: GraphileConfig.Plugin;
//# sourceMappingURL=PgColumnsPlugin.d.ts.map