"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgOrderCustomFieldsPlugin = void 0;
require("./PgTablesPlugin.js");
require("graphile-config");
const graphile_export_1 = require("graphile-export");
const behavior_js_1 = require("../behavior.js");
const version_js_1 = require("../version.js");
exports.PgOrderCustomFieldsPlugin = {
    name: "PgOrderCustomFieldsPlugin",
    description: "Adds ordering by 'computed column' functions",
    version: version_js_1.version,
    before: ["PgOrderAllColumnsPlugin"],
    inflection: {
        add: {
            computedColumnOrder(options, { source, variant }) {
                const computedColumnName = this.computedColumnField({ source });
                return this.constantCase(`${computedColumnName}-${variant}`);
            },
        },
    },
    schema: {
        hooks: {
            GraphQLEnumType_values(values, build, context) {
                const { inflection, sql } = build;
                const { scope: { isPgRowSortEnum, pgCodec }, } = context;
                if (!isPgRowSortEnum ||
                    !pgCodec ||
                    !pgCodec.columns ||
                    pgCodec.isAnonymous) {
                    return values;
                }
                const functionSources = build.input.pgSources.filter((source) => {
                    if (source.codec.columns)
                        return false;
                    if (source.codec.arrayOfCodec)
                        return false;
                    if (source.codec.rangeOfCodec)
                        return false;
                    const parameters = source.parameters;
                    if (!parameters || parameters.length < 1)
                        return false;
                    if (parameters.some((p, i) => i > 0 && p.required))
                        return false;
                    if (parameters[0].codec !== pgCodec)
                        return false;
                    if (!source.isUnique)
                        return false;
                    const behavior = (0, behavior_js_1.getBehavior)([
                        source.codec.extensions,
                        source.extensions,
                    ]);
                    // TODO: should this be `proc:orderBy`? If so, should we make it so `getBehavior` accepts a prefix to prepend, so `"orderBy"` in a smart tag on a proc becomes `proc:orderBy`?
                    return !!build.behavior.matches(behavior, "orderBy", "-orderBy");
                });
                return build.extend(values, functionSources.reduce((memo, pgFieldSource) => {
                    for (const ascDesc of ["asc", "desc"]) {
                        const valueName = inflection.computedColumnOrder({
                            source: pgFieldSource,
                            variant: ascDesc,
                        });
                        memo = build.extend(memo, {
                            [valueName]: {
                                extensions: {
                                    graphile: {
                                        applyPlan: (0, graphile_export_1.EXPORTABLE)((ascDesc, pgFieldSource, sql) => (step) => {
                                            if (typeof pgFieldSource.source !== "function") {
                                                throw new Error("Invalid computed column source");
                                            }
                                            const expression = sql `${pgFieldSource.source({
                                                placeholder: step.alias,
                                            })}`;
                                            step.orderBy({
                                                codec: pgFieldSource.codec,
                                                fragment: expression,
                                                direction: ascDesc.toUpperCase(),
                                            });
                                        }, [ascDesc, pgFieldSource, sql]),
                                    },
                                },
                            },
                        }, `Adding ascending orderBy enum value for ${pgCodec.name} from ${pgFieldSource}.`);
                    }
                    return memo;
                }, Object.create(null)), `Adding computed column orderable functions to order by for '${pgCodec.name}'`);
                return values;
            },
        },
    },
};
//# sourceMappingURL=PgOrderCustomFieldsPlugin.js.map