"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgRowByUniquePlugin = void 0;
const tslib_1 = require("tslib");
require("graphile-config");
const graphile_export_1 = require("graphile-export");
const tamedevil_1 = tslib_1.__importStar(require("tamedevil"));
const behavior_js_1 = require("../behavior.js");
const utils_js_1 = require("../utils.js");
const version_js_1 = require("../version.js");
exports.PgRowByUniquePlugin = {
    name: "PgRowByUniquePlugin",
    description: "Adds accessors for rows by their unique constraints (technically the @dataplan/pg data sources' 'uniques' property)",
    version: version_js_1.version,
    inflection: {
        add: {
            rowByUnique(options, { unique, source }) {
                if (typeof unique.extensions?.tags?.fieldName === "string") {
                    return unique.extensions?.tags?.fieldName;
                }
                const uniqueKeys = unique.columns;
                return this.camelCase(
                // TODO: should this use the _source_ rather than the _codec_ in case the same codec is used across multiple sources?
                `${this.tableType(source.codec)}-by-${this._joinColumnNames(source.codec, uniqueKeys)}`);
            },
        },
    },
    schema: {
        hooks: {
            GraphQLObjectType_fields(fields, build, context) {
                const { graphql: { GraphQLNonNull, GraphQLObjectType }, } = build;
                const { scope: { isRootQuery }, fieldWithHooks, } = context;
                if (!isRootQuery) {
                    return fields;
                }
                const sources = build.input.pgSources.filter((source) => {
                    if (source.parameters)
                        return false;
                    if (!source.codec.columns)
                        return false;
                    if (!source.uniques || source.uniques.length < 1)
                        return false;
                    return true;
                });
                return sources.reduce((outerMemo, source) => build.recoverable(outerMemo, () => source.uniques.reduce((memo, unique) => {
                    const uniqueKeys = unique.columns;
                    const fieldName = build.inflection.rowByUnique({
                        unique,
                        source,
                    });
                    const type = build.getTypeByName(build.inflection.tableType(source.codec));
                    if (!type || !(type instanceof GraphQLObjectType)) {
                        return memo;
                    }
                    const detailsByColumnName = Object.create(null);
                    uniqueKeys.forEach((columnName) => {
                        const column = source.codec.columns[columnName];
                        const columnArgName = build.inflection.column({
                            columnName,
                            codec: source.codec,
                        });
                        detailsByColumnName[columnName] = {
                            graphqlName: columnArgName,
                            codec: column.codec,
                        };
                    });
                    const columnNames = Object.keys(detailsByColumnName);
                    const clean = columnNames.every((key) => (0, tamedevil_1.isSafeObjectPropertyName)(key) &&
                        (0, tamedevil_1.isSafeObjectPropertyName)(detailsByColumnName[key].graphqlName));
                    const plan = clean
                        ? /*
                           * Since all the identifiers are nice and clean we can use
                           * an optimized function that doesn't loop over the
                           * attributes and just builds the object directly.  This is
                           * more performant, but it also makes the code nicer to
                           * read in the exported code.
                           */
                            // eslint-disable-next-line graphile-export/exhaustive-deps
                            (0, graphile_export_1.EXPORTABLE)(tamedevil_1.default.run `\
return function (source) {
  return (_$root, args) => source.get({ ${tamedevil_1.default.join(columnNames.map((columnName) => (0, tamedevil_1.default) `${tamedevil_1.default.dangerousKey(columnName)}: args.get(${tamedevil_1.default.lit(detailsByColumnName[columnName].graphqlName)})`), ", ")} });
}`, [source])
                        : (0, graphile_export_1.EXPORTABLE)((detailsByColumnName, source) => function plan(_$root, args) {
                            const spec = Object.create(null);
                            for (const columnName in detailsByColumnName) {
                                spec[columnName] = args.get(detailsByColumnName[columnName].graphqlName);
                            }
                            return source.get(spec);
                        }, [detailsByColumnName, source]);
                    const behavior = (0, behavior_js_1.getBehavior)([
                        source.codec.extensions,
                        source.extensions,
                        unique.extensions,
                    ]);
                    const fieldBehaviorScope = "query:source:single";
                    if (!build.behavior.matches(behavior, fieldBehaviorScope, "single")) {
                        return memo;
                    }
                    return build.extend(memo, {
                        [fieldName]: fieldWithHooks({
                            fieldName,
                            fieldBehaviorScope,
                        }, () => ({
                            description: `Get a single \`${type.name}\`.`,
                            deprecationReason: (0, utils_js_1.tagToString)(source.extensions?.tags?.deprecated),
                            type,
                            args: uniqueKeys.reduce((args, columnName) => {
                                const details = detailsByColumnName[columnName];
                                const columnType = build.getGraphQLTypeByPgCodec(details.codec, "input");
                                if (!columnType) {
                                    throw new Error(`Could not determine type for column`);
                                }
                                args[details.graphqlName] = {
                                    type: new GraphQLNonNull(columnType),
                                };
                                return args;
                            }, Object.create(null)),
                            plan: plan,
                        })),
                    }, `Adding row accessor for ${source} by unique columns ${uniqueKeys.join(",")}`);
                }, outerMemo)), fields);
            },
        },
    },
};
//# sourceMappingURL=PgRowByUniquePlugin.js.map